@@ -8,25 +8,23 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
     N <- length(dim(X))
     A <- list()
     length(A) <- N
-    for (n in 1:N) {
+    for (n in seq(N)) {
         if (init == "NMF") {
             Xn <- cs_unfold(X, m = n)@data
             res.nmf <- NMF(Xn, J = rank, algorithm = "KL")
             A[[n]] <- t(res.nmf$V)
-            orderA <- order(sapply(1:rank, function(x) {
+            orderA <- order(sapply(seq(rank), function(x) {
                 norm(as.matrix(res.nmf$V[, x], "F")) * norm(as.matrix(res.nmf$U[,
                   x]), "F")
             }), decreasing = TRUE)
-        }
-        else if (init == "ALS") {
+        } else if (init == "ALS") {
             Xn <- cs_unfold(X, m = n)@data
             res.svd <- svd(Xn)
-            A[[n]] <- .positive(res.svd$u[1:rank, ])
-            orderA <- order(sapply(res.svd$d[1:rank], function(x) {
+            A[[n]] <- .positive(res.svd$u[seq(rank), ])
+            orderA <- order(sapply(res.svd$d[seq(rank)], function(x) {
                 norm(as.matrix(x), "F")
             }), decreasing = TRUE)
-        }
-        else if (init == "Random") {
+        } else if (init == "Random") {
             A[[n]] <- matrix(runif(rank * dim(X)[n]), nrow = rank,
                 ncol = dim(X)[n])
             orderA <- order(apply(A[[n]], 1, function(x) {
@@ -40,18 +38,18 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
             }))
         }
     }
+
     iter <- 1
-    RecError = c() # Added in 2018.9.14
-    RelChange = c() # Added in 2018.9.14
-    X_bar <- recTensor(rep(1, length = rank), A)
+    RecError = c()
+    RelChange = c()
+    X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
     RecError[1] <- .recError(X, X_bar)
     RelChange[1] <- thr * 10
     if (algorithm == "HALS") {
-        T1 <- (A[[1]] %*% t(A[[1]])) * (A[[2]] %*% t(A[[2]])) *
-            (A[[3]] %*% t(A[[3]]))
+        T1 <- eval(parse(text=.HALSCMD8(N)))
     }
     if (algorithm == "Alpha-HALS" || algorithm == "Beta-HALS") {
-        X_bar <- recTensor(rep(1, length = rank), A)
+        X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
         E <- X - X_bar
     }
     if (algorithm == "Frobenius") {
@@ -83,15 +81,13 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
     }
     while ((RelChange[iter] > thr) && (iter <= num.iter)) {
         # Before Update U, V
-        X_bar <- recTensor(rep(1, length = rank), A)
+        X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
         pre_Error <- .recError(X, X_bar)
         if (algorithm == "Alpha") {
-            for (n in 1:N) {
-                X_bar <- recTensor(rep(1, length = rank), A)
-                tmp1 <- A[[setdiff(1:rank, n)[1]]]
-                tmp2 <- A[[setdiff(1:rank, n)[2]]]
-                A_nonn <- khatri_rao(t(tmp1), t(tmp2))
-                A[[n]] <- A[[n]] * (t(A_nonn) %*% (cs_unfold(X,
+            for (n in seq(N)) {
+                X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
+                A_notn <- .KhatriRao_notn(A, n)
+                A[[n]] <- A[[n]] * (t(A_notn) %*% (cs_unfold(X,
                   m = n)@data/cs_unfold(X_bar, m = n)@data)^Alpha)^(1/Alpha)
                 if (n != N) {
                   A[[n]] <- t(apply(A[[n]], 1, function(x) {
@@ -101,14 +97,11 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
             }
         }
         else if (algorithm == "Beta") {
-            X_bar <- recTensor(rep(1, length = rank), A)
-            for (n in 1:N) {
-                tmp1 <- A[[setdiff(1:rank, n)[1]]]
-                tmp2 <- A[[setdiff(1:rank, n)[2]]]
-                A_nonn <- khatri_rao(t(tmp1), t(tmp2))
-                numer <- t(A_nonn) %*% (cs_unfold(X, m = n)@data/cs_unfold(X_bar^(Beta -
-                  1), m = n)@data)
-                denom <- t(A_nonn) %*% cs_unfold(X_bar^(Beta),
+            X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
+            for (n in seq(N)) {
+                A_notn <- .KhatriRao_notn(A, n)
+                numer <- t(A_notn) %*% (cs_unfold(X, m = n)@data/cs_unfold(X_bar^(Beta - 1), m = n)@data)
+                denom <- t(A_notn) %*% cs_unfold(X_bar^(Beta),
                   m = n)@data
                 A[[n]] <- A[[n]] * numer/denom
                 if (n != N) {
@@ -120,16 +113,14 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
         }
         else if (algorithm == "HALS") {
             gamma <- diag((A[[N]] %*% t(A[[N]])))
-            for (n in 1:N) {
+            for (n in seq(N)) {
                 if (n == N) {
                   gamma[] <- 1
                 }
-                tmp1 <- A[[setdiff(1:rank, n)[1]]]
-                tmp2 <- A[[setdiff(1:rank, n)[2]]]
-                A_nonn <- khatri_rao(t(tmp1), t(tmp2))
-                T2 = t(cs_unfold(X, m = n)@data) %*% A_nonn
+                A_notn <- .KhatriRao_notn(A, n)
+                T2 = t(cs_unfold(X, m = n)@data) %*% A_notn
                 T3 = T1/(A[[n]] %*% t(A[[n]]))
-                for (r in 1:rank) {
+                for (r in seq(rank)) {
                   A[[n]][r, ] = .positive(gamma[r] * A[[n]][r,
                     ] + T2[, r] - as.vector(t(A[[n]]) %*% T3[,
                     r]))^2
@@ -142,22 +133,21 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
             }
         }
         else if (algorithm == "Alpha-HALS") {
-            for (r in 1:rank) {
-                X_bar <- recTensor(rep(1, length = rank), A)
+            for (r in seq(rank)) {
+                X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
                 Xr <- E + X_bar
-                for (n in 1:N) {
+                for (n in seq(N)) {
                   if (Alpha == 0) {
                     tmp_Xr <- .positive(Xr)
-                    tmp_Xr@data[, , ] <- log(tmp_Xr@data[, ,
-                      ])
+                    tmp_Xr@data <- log(tmp_Xr@data)
                     numer <- tmp_Xr
                   }
                   else {
                     numer <- .positive(Xr)^Alpha
                   }
                   denom <- 0
-                  non_n <- setdiff(1:N, n)
-                  for (m in non_n) {
+                  not_n <- setdiff(seq(N), n)
+                  for (m in not_n) {
                     numer <- ttm(numer, t(as.matrix(A[[m]][r,
                       ])), m = m)
                     if (Alpha == 0) {
@@ -181,18 +171,18 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
                       ]), "F")
                   }
                 }
-                X_bar <- recTensor(rep(1, length = rank), A)
+                X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
                 E <- Xr - X_bar
             }
         }
         else if (algorithm == "Beta-HALS") {
-            for (r in 1:rank) {
-                X_bar <- recTensor(rep(1, length = rank), A)
+            for (r in seq(rank)) {
+                X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
                 Xr <- E + X_bar
                 for (n in 1:(N - 1)) {
-                  non_n <- setdiff(1:N, n)
+                  not_n <- setdiff(seq(N), n)
                   tmp_u <- Xr
-                  for (m in non_n) {
+                  for (m in not_n) {
                     tmp_u <- ttm(tmp_u, t(as.matrix((A[[m]][r,
                       ])))^Beta, m = m)
                   }
@@ -200,39 +190,39 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
                   A[[n]][r, ] <- A[[n]][r, ]/norm(as.matrix(A[[n]][r,
                     ]), "F")
                 }
-                non_N <- setdiff(1:N, N)
+                not_N <- setdiff(seq(N), N)
                 numer <- Xr
                 denom <- 0
-                for (m in non_N) {
+                for (m in not_N) {
                   numer <- ttm(numer, t(as.matrix(A[[m]][r, ])),
                     m = m)
                   denom <- denom + as.numeric(t(A[[m]][r, ])^Beta %*%
                     A[[m]][r, ])
                 }
                 A[[N]][r, ] <- .positive(as.vector(numer@data[]/denom))
-                X_bar <- recTensor(rep(1, length = rank), A)
+                X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
                 E <- Xr - X_bar
             }
         }
         else {
             stop("Please specify the appropriate algorithm\n")
         }
-        for (n in 1:N) {
+        for (n in seq(N)) {
             if (any(is.infinite(A[[n]])) || any(is.nan(A[[n]]))) {
                 stop("Inf or NaN is generated!\n")
             }
         }
         # After Update U, V
         iter <- iter + 1
-        X_bar <- recTensor(rep(1, length = rank), A)
+        X_bar <- recTensor(rep(1, length = rank), A, idx=seq(N))
         RecError[iter] <- .recError(X, X_bar)
         RelChange[iter] <- abs(pre_Error - RecError[iter]) / RecError[iter]
-        if (viz && !is.null(figdir)) {
+        if (viz && !is.null(figdir) && N == 3) {
             png(filename = paste0(figdir, "/", iter, ".png"))
             plotTensor3D(X_bar)
             dev.off()
         }
-        if (viz && is.null(figdir)) {
+        if (viz && is.null(figdir) && N == 3) {
             plotTensor3D(X_bar)
         }
         if (verbose) {
@@ -243,7 +233,7 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
             stop("NaN is generated. Please run again or change the parameters.\n")
         }
     }
-    if (viz && !is.null(figdir)) {
+    if (viz && !is.null(figdir) && N == 3) {
         png(filename = paste0(figdir, "/finish.png"))
         plotTensor3D(X_bar)
         dev.off()
@@ -251,10 +241,21 @@ NTF <- function(X, rank = 3, algorithm = "KL", init = "NMF", Alpha = 1,
         plotTensor3D(X)
         dev.off()
     }
-    if (viz && is.null(figdir)) {
+    if (viz && is.null(figdir) && N == 3) {
         plotTensor3D(X_bar)
     }
     names(RecError) <- c("offset", 1:(iter-1))
     names(RelChange) <- c("offset", 1:(iter-1))
-    return(list(S = .diag(X_bar), A = A, RecError = RecError, RelChange = RelChange))
+    # normalization
+    S <- apply(A[[N]], 1, function(an){
+        norm(as.matrix(an), "F")
+    })
+    A[[N]] <- A[[N]] / S
+    # sort
+    orderS <- order(S, decreasing=TRUE)
+    S <- S[orderS]
+    for(n in seq(N)){
+        A[[n]] <- A[[n]][orderS, ]
+    }
+    return(list(S = S, A = A, RecError = RecError, RelChange = RelChange))
 }
